package cz.muni.fi.pb138.fsmda.analysis;

import cz.muni.fi.pb138.fsmda.common.AnalysisException;
import cz.muni.fi.pb138.fsmda.comparison.ComparisonManagerImpl;
import java.util.ArrayDeque;
import java.util.Stack;
import java.util.logging.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Implementation of DirectorySizer.
 *
 * @author Matus Nemec
 */
public class DirectorySizerImpl implements DirectorySizer {

    static final Logger logger = Logger.getLogger(ComparisonManagerImpl.class.getName());

    /**
     * Builds a stack containing nodes using BFS, to assure correct order of
     * operations. Size of leaves must be computed first, than it is possible to
     * move one level up, until the root directory is reached and its size is
     * computed.
     */
    @Override
    public void addSizeOfDirectories(Document doc) {
        logger.info("Starting directory size computation");
        Stack<Node> stack = new Stack<Node>();
        ArrayDeque<Node> queue = new ArrayDeque<Node>();
        Node node = doc.getFirstChild();
        if (node == null) {
            logger.severe("Document has no root element");
            throw new AnalysisException("Document has no root element");
        }
        stack.push(node);
        queue.add(node);
        while ((!queue.isEmpty()) && ((node = queue.removeFirst()) != null)) {
            NodeList children = node.getChildNodes();
            if (children == null) {
                continue;
            }
            for (int i = 0; i < children.getLength(); i++) {
                if ("directory".equals(children.item(i).getNodeName())) {
                    stack.push(children.item(i));
                    queue.add(children.item(i));
                }
            }
        }
        while ((!stack.isEmpty()) && ((node = stack.pop()) != null)) {
            Long length = new Long(0);
            NodeList children = node.getChildNodes();
            for (int i = 0; i < children.getLength(); i++) {
                if (((Element) children.item(i)).getAttribute("length") != null) {
                    length += Long.parseLong(((Element) children.item(i)).getAttribute("length"));
                }
            }
            ((Element) node).setAttribute("length", String.valueOf(length));
        }
        logger.info("Directory size computation complete");
    }
}
