/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.pb138.fsmda.analysis;

import cz.muni.fi.pb138.fsmda.common.ValidationException;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import javax.xml.xpath.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 *
 * @author Adriana
 */
public class DomManagerImpl implements DomManager{ 

    private String fileSystemName = "null";
    //private Document dom;
    private Validator validator;
    private static final Logger logger = Logger.getLogger(DomManagerImpl.class.getSimpleName());

    public DomManagerImpl() throws SAXException{
        try {
            SchemaFactory factory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");
            File schemaLocation = new File("schema.xsd");
            Schema schema = factory.newSchema(schemaLocation);
            validator = schema.newValidator();
        } catch (SAXException ex) {
            logger.severe("Error: Building schema failed "+ex);
            throw ex;
        }
    }

    @Override
    public void createXML(Document dom) throws TransformerConfigurationException, TransformerException {
        try {
            logger.info("Started creating XML from Document");
            String name = fileSystemName + time("yyyy_MM_dd_H_mm_ss_SSS",0l) + ".xml";
            Source source = new DOMSource(dom);
            if(!(new File("history").exists())){
                new File("history").mkdir();
            }
            File file = new File("history/" + name);
            Result result = new StreamResult(file);

            Transformer xformer = TransformerFactory.newInstance().newTransformer();
            xformer.transform(source, result);
            logger.info("Creating complete");
        } catch (TransformerConfigurationException ex) {
            logger.severe("Error: Creating XML failed "+ex);
            throw ex;
        } catch(TransformerException ex){
            logger.severe("Error: Creating XML failed "+ex);
            throw ex;
        }
    }

    @Override
    public Document domFromXML(String path) throws SAXException, IOException, ParserConfigurationException {

        File xmlFile = new File(path);
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        DocumentBuilder db = dbf.newDocumentBuilder();
        Document doc = db.parse(xmlFile);
        validation(doc);

        return doc;
    }
    
    /**
     * Creates string from date
     * @param dateFormat string representing format of date 
     * @param time long representing time which is transforming
     */

    public static String time(String dateFormat,long time) {
        Calendar cal = Calendar.getInstance();
        if(time!=0l) cal.setTimeInMillis(time);
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        
        return sdf.format(cal.getTime());

    }
    
    public String getFileSystemName() {
        return fileSystemName;
    }

    public void setFileSystemName(String fileSystemName) {
        if(fileSystemName == null) throw new IllegalArgumentException();
        this.fileSystemName = fileSystemName;
    }
    
    
    @Override
    public void validation(Document document) {
        logger.info("Started validating Document");
        try {
            validator.validate(new DOMSource(document));
            logger.info("Completed validating Document");
        } catch (SAXException ex) {
            logger.severe("Validation failed "+ex);
            throw new ValidationException(ex);
        } catch (IOException ex) {
            logger.severe("Validation failed "+ex);
            throw new ValidationException(ex);
        }
    }
}