package cz.muni.fi.pb138.fsmda.analysis;

import cz.muni.fi.pb138.fsmda.common.AnalysisException;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayDeque;
import java.util.Calendar;
import java.util.Queue;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 *
 * @author Maros Klimovsky
 */

/**
 * Implementation of FileSystemAnalyser
 * 
 * @author Maros
 */
public class FileSystemAnalyserImpl implements FileSystemAnalyser {
    Document document;
     private Queue<Node> filesQueue = new ArrayDeque();
    private static final Logger logger = Logger.getLogger(FileSystemAnalyserImpl.class.getSimpleName());
    DomManagerImpl manager = null;
    
    public FileSystemAnalyserImpl() throws IOException, InterruptedException {
             
        try {
         
        manager = new DomManagerImpl();
        }
        catch (SAXException ex) {
             logger.severe("Error: Analysis failed "+ex);
            throw new AnalysisException("Error: Can't perform analysis. Check log file for more info.");
        }
    }
    
    /**
     * Method analyzes given file system starting from given rootPath.
     * Root directory itself is not included and is represented only as <fileSystem> element
     * and creates DOM document with result
     * 
     * @param rootPath - path to root directory of file system on disc
     * @return Document containing XML representation of file system
     */
    @Override
    public Document analyseFileSystem(String rootPath) {
        logger.info("Starting analysis");
        if ((rootPath==null)||(rootPath.equals(""))) {
            logger.warning("Null path to root ");
            throw new AnalysisException("Root path can't be null!");
        }
        try {
            document = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        } catch (ParserConfigurationException ex) {
            logger.severe("Error: Analysis failed "+ex);
                throw new AnalysisException("Error: Can't perform analysis. Check log file for more info.");
            
        }
        
        File root = new File(rootPath);
        if (!root.isDirectory()) {
            logger.warning("Root "+rootPath+" is NOT a directory.");
            throw new AnalysisException("Root must be directory!");
        }
        Element rootElement = document.createElement("fileSystem");
        addChildrenToQueue(root, rootElement);
        document.appendChild(rootElement);
        manager.setFileSystemName(root.getName());
        Node node;
        while ((node =filesQueue.poll()) != null) {
              addChildrenToQueue(new File(((Element)node).getAttribute("name")), node);   
                   
        }
        
      
       ((Element)document.getFirstChild()).removeAttribute("name");
        new DirectorySizerImpl().addSizeOfDirectories(document);
      
       
        logger.info("Analysis complete");
        return document;
    }
    
       private void addChildrenToQueue(File file,Node parentNode) {
           String tagName = null;
           Boolean directory = false;
           Element element = null;
           File[] files;
                   
           ((Element)parentNode).setAttribute("name", file.getName());
           if (((files = file.listFiles())!=null)) {
                for (File f : files ) {
                    if ((directory=f.isDirectory())) {
                        tagName = "directory";
                    }
                    else{ 
                        tagName = "file";
                    }
                    
                    element = document.createElement(tagName);
                    element.setAttribute("lastModified", DomManagerImpl.time("dd.MM.yyyy|HH:mm:ss",f.lastModified()));
        if(f.isHidden()) element.setAttribute("hidden", "true");
        if (!directory) {
            element.setAttribute("name", f.getName());
            element.setAttribute("length", Long.toString(f.length()));
        }
        else{
            element.setAttribute("name", f.getAbsolutePath());
            element.setAttribute("length", "0");
        }
        
        
        element.setAttribute("readable", ((Boolean)(f.canRead())).toString());
        element.setAttribute("writable", ((Boolean)(f.canWrite())).toString());
        element.setAttribute("executable", ((Boolean)(f.canExecute())).toString());
                 
                    parentNode.appendChild(element);
                    if (directory) filesQueue.add(element);
                }
                
            }
        }
         
    /**
     * Method analyzes given file system starting from given rootPath.
     * Root directory itself is not included and is represented only as <fileSystem> element
     * After successful analysis it creates XML file with result
     * 
     * @param rootPath - path to root directory of file system on disc
     */
    @Override
    public void analyseFileSystemAndStore(String rootPath) {
        Document document = analyseFileSystem(rootPath);
        try {
            manager.createXML(document);
        } catch (TransformerConfigurationException ex) {
           logger.severe("Error: Analysis failed "+ex);
                throw new AnalysisException("Error: Can't perform analysis. Check log file for more info.");
        } catch (TransformerException ex) {
            logger.severe("Error: Analysis failed "+ex);
                throw new AnalysisException("Error: Can't perform analysis. Check log file for more info.");
        }
    }
    
}
