
package cz.muni.fi.pb138.fsmda.comparison;

import cz.muni.fi.pb138.fsmda.analysis.DomManagerImpl;
import cz.muni.fi.pb138.fsmda.analysis.FileSystemAnalyserImpl;

import cz.muni.fi.pb138.fsmda.common.ValidationException;
import java.io.IOException;
import java.util.List;
import java.util.logging.Logger;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 *  Implementation of ComparisonManager
 * @author Maros
 */
public class ComparisonManagerImpl implements ComparisonManager {
    DomManagerImpl domManager;
    static final Logger logger = Logger.getLogger(ComparisonManagerImpl.class.getName());
    
    /**
     * Takes analysis of current state on disc and validated XML file converted
     * to Document, compares it and returns result. 
     * 
     */
    public ComparisonManagerImpl() {
        
        try {
            
                domManager = new DomManagerImpl();
            } catch (SAXException ex) {
                logger.severe("Error: Comparison failed "+ex);
                throw new RuntimeException("Comparision failed! Check log file for more info.");
            
        } 
    }
    
    /**
     * Method prepares documents for comparison and returns result given by FileSystemComparer
     * 
     * @param filePath - path to xml file
     * @param fileSystemPath - path to file system
     * @return list of differencies
     */
    @Override
    public List<String> compare(String filePath, String fileSystemPath) {
        logger.info("Preparing comparison");
        if ((filePath==null)||("".equals(filePath))) {
        logger.warning("filePath was null ");
        throw new IllegalArgumentException("filePath not set!");
        
    }
        if ((filePath==null)||("".equals(fileSystemPath))) {
            logger.warning("filePath was null ");
            throw new IllegalArgumentException("fileSystemPath not set!");
    }
        
            Document externalFile = null;
            try {
              externalFile   = domManager.domFromXML(filePath);
              if (externalFile==null) {
                  logger.severe("Retrieving of file system "+filePath+" returned null!");
                  throw new RuntimeException("Comparision failed! Check log file for more info.");
              }
            } catch (SAXException ex) {
                logger.severe("Error: Comparison failed "+ex);
                throw new RuntimeException("Comparision failed! Check log file for more info.");
            } catch (IOException ex) {
                logger.severe("Error: Comparison failed "+ex);
                throw new RuntimeException("Comparision failed! Check log file for more info.");
            } catch (ParserConfigurationException ex) {
                logger.severe("Error: Comparison failed "+ex);
                throw new RuntimeException("Comparision failed! Check log file for more info.");
            }
            Document analysisResult = null;
        try {
            
            analysisResult = new FileSystemAnalyserImpl().analyseFileSystem(fileSystemPath);
            if (analysisResult==null) {
                  logger.severe("Analysis of file system "+fileSystemPath+" returned null!");
                  throw new RuntimeException("Comparision failed! Check log file for more info.");
              }
        } catch (IOException ex) {
            logger.severe("Error: Comparison failed "+ex);
                throw new RuntimeException("Comparision failed! Check log file for more info.");
        } catch (InterruptedException ex) {
            logger.severe("Error: Comparison failed "+ex);
                throw new RuntimeException("Comparision failed! Check log file for more info.");
        }
       
        
       List<String> list = new FileSystemComparerImpl().compareFileSystemRepresentations(externalFile, analysisResult); 
       if (list==null) {
                  logger.severe("Comparison returned null!");
                  throw new RuntimeException("Comparision failed! Check log file for more info.");
              }
       logger.info("Comparison completed");
       return list; 
    }
    
}
