package cz.muni.fi.pb138.fsmda.gui;

import cz.muni.fi.pb138.fsmda.analysis.FileSystemAnalyser;
import cz.muni.fi.pb138.fsmda.analysis.FileSystemAnalyserImpl;
import cz.muni.fi.pb138.fsmda.common.AnalysisException;
import java.awt.Component;
import java.io.File;
import java.io.IOException;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;

/**
 * This class is insuring thread security by extending SwingWorker.
 * It's calling method from FileSystemAnalyser class and after it's done
 * it'll call abstract method finishAnalysis()
 * 
 * @author Radoslav Rabara
 */
public abstract class AnalyzeThread extends SwingWorker<String, Object> {
    private Component parent;
    private File selectedFolder = null;
    private FileSystemAnalyser fileSystemAnalyser;
    private static final String errorMessageTitle = "Error while inicializing file system analyser!";
    private Logger logger;
    /**
     * @param parent is used as parent in JOptionPane when we need to show some ERROR/WARNING message
     * @param logger from MainJFrame is shared by all graphics components
     * @param selectedFolder from which will be done analysis
     */
    public AnalyzeThread(Component parent,Logger logger,File selectedFolder) {
        this.parent = parent;
        this.selectedFolder = selectedFolder;
        this.logger = logger;
        try{
            fileSystemAnalyser = new FileSystemAnalyserImpl();
        } catch(IOException ioe){
            logger.fine("IOException while inicializing file system analyser!");
            JOptionPane.showMessageDialog(parent,"File system analyser can't be inicialized due to IO exception!",errorMessageTitle,JOptionPane.ERROR_MESSAGE);
        } catch(InterruptedException ie){
            logger.severe("InterruptedException while inicializing file system analyser!");
            JOptionPane.showMessageDialog(parent,"File system analyser can't be inicialized due to Interrupted exception!",errorMessageTitle,JOptionPane.ERROR_MESSAGE);
        } catch(AnalysisException ae){
            logger.severe("AnalysisException while inicializing file system analyser!");
            JOptionPane.showMessageDialog(parent,"File system analyser can't be inicialized due to internal exception!",errorMessageTitle,JOptionPane.ERROR_MESSAGE);
        } catch(Exception e){
            logger.severe("Unexpected exception while inicializing comparison manager.");
            JOptionPane.showMessageDialog(parent,"File system analyser can't be inicialized due to exception!",errorMessageTitle,JOptionPane.ERROR_MESSAGE);
        }
    }

    @Override
    public String doInBackground() {
        if(fileSystemAnalyser==null)
            return "File system analyser wasn't inicialized!";
        try{
            fileSystemAnalyser.analyseFileSystemAndStore(selectedFolder.getAbsolutePath());
        }catch(AnalysisException ae){
            return ae.getMessage();
        }
        return "Analysis was successful!";
    }

    @Override
    protected void done() {
        if(this.isCancelled())
            return;
        String get;
        try {
            get = get();
            finishAnalysis();
            JOptionPane.showMessageDialog(parent, get, "Result of analysis", JOptionPane.INFORMATION_MESSAGE);
        } catch (InterruptedException ex) {
            logger.severe(ex.getMessage());
        } catch (ExecutionException ex) {
            logger.severe(ex.getMessage());
        } catch(CancellationException ex){
            logger.severe(ex.getMessage());
        }  catch(Exception e){
            logger.severe(e.getMessage());
        }      
    }
    /**
     This method is called after analysis is finished.
     */
    public abstract void finishAnalysis();
}
