package cz.muni.fi.pb138.fsmda.gui;

import cz.muni.fi.pb138.fsmda.comparison.ComparisonManager;
import cz.muni.fi.pb138.fsmda.comparison.ComparisonManagerImpl;
import java.awt.Component;
import java.io.File;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.logging.Logger;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;

/**
 * This class is insuring thread security by extending SwingWorker.
 * It's calling method from ComparisonManager class and after it's done
 * it'll call abstract method finishAnalysis().
 * This class has the same design as AnalyseThread class.
 * 
 * @author Radoslav Rabara
 */
public abstract class CompareThread extends SwingWorker<List<String>,Object>{
    private Component parent;
    private ComparisonManager comparisionManager;
    private File mainFolder,selectedXMLfile;
    private Logger logger;
    /**
     * @param parent is used as parent in JOptionPane when we need to show some ERROR/WARNING message 
     * @param logger from MainJFrame is shared by all graphics components
     * @param mainFolder represents actual state of disc
     * @param selectedXMLfile will be compared with mainFolder
     */
    public CompareThread(Component parent,Logger logger,File mainFolder,File selectedXMLfile){
        this.parent = parent;
        this.mainFolder = mainFolder;
        this.selectedXMLfile = selectedXMLfile;
        this.logger = logger;
        try{
            comparisionManager = new ComparisonManagerImpl();
        }catch(RuntimeException runtimeException){
            logger.severe("RuntimeException while inicializing comparison manager.");
            JOptionPane.showMessageDialog(parent,runtimeException.getMessage(),"Error while comparing files",JOptionPane.ERROR_MESSAGE);
        }catch(Exception e){
            logger.severe("Unexpected exception while inicializing comparison manager.");
            JOptionPane.showMessageDialog(parent,e.getMessage(),"Error while comparing files",JOptionPane.ERROR_MESSAGE);
        }
    }
    @Override
    protected List<String> doInBackground() throws Exception {        
         List<String> list = new LinkedList<String>();
        try{
            list = comparisionManager.compare(selectedXMLfile.getAbsolutePath(), mainFolder.getAbsolutePath());            
        }catch(IllegalArgumentException iae){
            logger.severe("IllegalArgumentException while running comparison manager.");
            list.add(iae.getMessage());
        }catch(RuntimeException re){
            logger.severe("RuntimeException while running comparison manager.");
            list.add(re.getMessage());
        }catch(Exception e){
            logger.severe("Unexpected exception while inicializing comparison manager.");
            list.add(e.getMessage());
        }
        return list;
    }
    @Override
    protected void done() {
        if(this.isCancelled())
            return;        
        try {
            List<String> get = get();
            finishAnalysis();
            if(get.size()==0)
                JOptionPane.showMessageDialog(parent, "Comaparison was successful!\nDocuments are identical!!!", "Result of comparation", JOptionPane.INFORMATION_MESSAGE);                
            else{
                new ResultOfComparationDialog((JFrame)parent,mainFolder.getAbsolutePath(),selectedXMLfile.getAbsolutePath(),get);
            }
        } catch (InterruptedException ex) {
            logger.severe(ex.getMessage());
        } catch (ExecutionException ex) {
            logger.severe(ex.getMessage());
        } catch(CancellationException ex){
            logger.severe(ex.getMessage());
        }
    }
    /**
     * This method is called after comparision is finished.
     */
    public abstract void finishAnalysis();
}
