package cz.muni.fi.pb138.fsmda.gui;

/**
 * TableModel with two columns: Location and Modified.
 * Is's containing set of files which are xml files with file disc analysis.
 * 
 * @author Radoslav Rabara
 */
import java.io.File;
import java.text.DateFormat;
import java.util.*;
import java.util.logging.Logger;
import javax.swing.table.AbstractTableModel;

public class HistoryTableModel extends AbstractTableModel{
    private Set<File> set = new TreeSet<File>(new FileComparator());
    private static final DateFormat formatter =  DateFormat.getDateInstance(DateFormat.LONG);
    private Logger logger;
    public HistoryTableModel(Logger logger){
        this.logger = logger;
        refreshTableModel();
    }
    @Override
    public int getRowCount() {
        return set.size();
    }
    @Override
    public int getColumnCount() {
        return 2;
    }
    @Override
    public String getColumnName(int column){
        if(column==0)
            return "Location";
        return "Modified";
    }
    @Override
    public Object getValueAt(int row, int column) {
        File file = (File) set.toArray()[row];
        if(column==0)
            return file.getPath();
        Date yourDate = new Date(file.lastModified());
        return formatter.format(yourDate);
    }
    /**
     * Add your own file to table model
     * @param file should be xml file with disc analysis
     * @throws IllegalArgumentException when added file was already in table
     */
    public void addFile(File file) throws IllegalArgumentException{
        if(set.contains(file)==true)
            throw new IllegalArgumentException("Adding file was already in table!");
        set.add(file);
        this.fireTableRowsInserted(set.size()-1,set.size()-1);
    } 
    /**
     * Get selected folder by index from table.
     * @param index of file which will be returned
     * @return filde with index which was given as parameter
     */
    public File getFile(int index){
        return (File) set.toArray()[index];
    }
    /**
     * It'll check if any files was added to 'history' folder in which are stored
     * xml files with disc analysis.
     */
    public void refreshTableModel(){
        File historyFolder = new File("history");
        if(historyFolder.exists()){
            try{
                File[] files = historyFolder.listFiles();
                set.addAll(Arrays.asList(files));
                this.fireTableDataChanged();
            }catch(Exception e){
                logger.severe("Exception while loading files from history file!");
            }
        }
    }
}
