package cz.muni.fi.pb138.fsmda.gui;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.logging.LogManager;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
/**
 * Main class which creates main window with two tabs ('Analyse selected folder' and 'Compare').
 * 
 * In first tab ('Analyse selected folder') there are three buttons.
 * First button for folder selection, from which will be done analysis.
 * There are also labels which gives information about selected folder's path.
 * Next buttons are 'Analyse' and 'Cancel'. There are also progress bar (in indeterminated
 * state which informs about doing analysis) and labels with icons for more friendly GUI.
 * 
 * In second tab ('Compare') there is also button and labels for folder selection,
 * from which will be done analysis for comparation with selected xml file,
 * which is selected in 'history table'(specified later in this text).
 * Under this is 'history table' which contains xml files with disc analysis
 * (xml file with disc specification is created after analysis from first tab).
 * Under 'history table' there are two buttons:
 * - first for loading user's own xml file with disc analysis
 * - second for starting comparation
 * There are also progress bar (also in indeterminated state), which informs about
 * doing comparation, cancel button and labels for more friendly GUI.
 * 
 * @author Radoslav Rabara
 */
public class MainJFrame extends javax.swing.JFrame {
    private Logger logger = Logger.getLogger(MainJFrame.class.getName());
    /**
     * Creates new form MainJFrame
     */
    public MainJFrame() {
        setLogger();
        logger.info("Starting application");
        initComponents();
        historyTable.setRowHeight(26);
        historyTable.getColumnModel().getColumn(0).setPreferredWidth(200);
        /*
         * historyTable.getSelectionModel().addListSelectionListener(new
         * ListSelectionListener(){ public void valueChanged(ListSelectionEvent
         * lse) { if(lse.getValueIsAdjusting()) return;
         * SwingUtilities.invokeLater(new Runnable(){ public void run() {
         * compareButton.setEnabled(true); } }); }
         *
         * });
         */
    }

    private static void setLogger() {

        try {
            FileInputStream configFile = new FileInputStream("log.properties");
            /*
             * for (Handler h: Logger.getLogger("").getHandlers()) {
             * Logger.getLogger("").removeHandler(h); }
             */
            LogManager.getLogManager().readConfiguration(configFile);
        } catch (IOException ex) {
            System.err.println("WARNING: Could not open configuration file");
            System.err.println("WARNING: Logging not configured (console output only)");
        }


    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel2 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        comparisionTab = new javax.swing.JTabbedPane();
        analysisSelectedFolder = new javax.swing.JPanel();
        chooseFolderButton = new javax.swing.JButton();
        doAnalysisButton = new javax.swing.JButton();
        selectedFolderLabel = new javax.swing.JLabel();
        analysisProgressBar = new javax.swing.JProgressBar();
        cancelAnalysisButton = new javax.swing.JButton();
        selectedFolderText = new javax.swing.JLabel();
        analyseIcon = new javax.swing.JLabel();
        folderIcon = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        historyTable = new javax.swing.JTable();
        loadYourOwnFile = new javax.swing.JButton();
        compareButton = new javax.swing.JButton();
        xmlIcon = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        compareProgressBar = new javax.swing.JProgressBar();
        cancelCompareButton = new javax.swing.JButton();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        selectedFolderCompareLabel = new javax.swing.JLabel();
        selectFolderCompareButton = new javax.swing.JToggleButton();

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 100, Short.MAX_VALUE)
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 100, Short.MAX_VALUE)
        );

        jLabel2.setText("jLabel2");

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("File System Meta Data Analysis");

        chooseFolderButton.setText("Select folder");
        chooseFolderButton.setNextFocusableComponent(doAnalysisButton);
        chooseFolderButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                chooseFolderButtonActionPerformed(evt);
            }
        });

        doAnalysisButton.setText("Analyse");
        doAnalysisButton.setEnabled(false);
        doAnalysisButton.setNextFocusableComponent(cancelAnalysisButton);
        doAnalysisButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                doAnalysisButtonActionPerformed(evt);
            }
        });

        selectedFolderLabel.setForeground(new java.awt.Color(102, 102, 102));
        selectedFolderLabel.setText("Selected folder:");

        analysisProgressBar.setEnabled(false);

        cancelAnalysisButton.setText("Cancel");
        cancelAnalysisButton.setEnabled(false);
        cancelAnalysisButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelAnalysisButtonActionPerformed(evt);
            }
        });

        analyseIcon.setIcon(new javax.swing.ImageIcon(getClass().getResource("/cz/muni/fi/pb138/fsmda/gui/analyze.png"))); // NOI18N

        folderIcon.setIcon(new javax.swing.ImageIcon(getClass().getResource("/cz/muni/fi/pb138/fsmda/gui/folder.png"))); // NOI18N

        javax.swing.GroupLayout analysisSelectedFolderLayout = new javax.swing.GroupLayout(analysisSelectedFolder);
        analysisSelectedFolder.setLayout(analysisSelectedFolderLayout);
        analysisSelectedFolderLayout.setHorizontalGroup(
            analysisSelectedFolderLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(analysisSelectedFolderLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(analysisSelectedFolderLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(analysisSelectedFolderLayout.createSequentialGroup()
                        .addComponent(analyseIcon)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(analysisSelectedFolderLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(doAnalysisButton, javax.swing.GroupLayout.PREFERRED_SIZE, 98, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(analysisProgressBar, javax.swing.GroupLayout.DEFAULT_SIZE, 305, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(cancelAnalysisButton))
                    .addGroup(analysisSelectedFolderLayout.createSequentialGroup()
                        .addComponent(folderIcon)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(analysisSelectedFolderLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(analysisSelectedFolderLayout.createSequentialGroup()
                                .addComponent(chooseFolderButton)
                                .addGap(0, 0, Short.MAX_VALUE))
                            .addGroup(analysisSelectedFolderLayout.createSequentialGroup()
                                .addComponent(selectedFolderLabel)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(selectedFolderText, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))))
                .addContainerGap())
        );
        analysisSelectedFolderLayout.setVerticalGroup(
            analysisSelectedFolderLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(analysisSelectedFolderLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(analysisSelectedFolderLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(folderIcon)
                    .addGroup(analysisSelectedFolderLayout.createSequentialGroup()
                        .addGap(28, 28, 28)
                        .addComponent(chooseFolderButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(analysisSelectedFolderLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(selectedFolderLabel)
                            .addComponent(selectedFolderText, javax.swing.GroupLayout.PREFERRED_SIZE, 14, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addGap(18, 18, 18)
                .addGroup(analysisSelectedFolderLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(analysisSelectedFolderLayout.createSequentialGroup()
                        .addComponent(doAnalysisButton)
                        .addGap(18, 18, 18)
                        .addGroup(analysisSelectedFolderLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(analysisProgressBar, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(cancelAnalysisButton, javax.swing.GroupLayout.PREFERRED_SIZE, 24, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(analyseIcon))
                .addContainerGap(132, Short.MAX_VALUE))
        );

        comparisionTab.addTab("Analyse selected folder", analysisSelectedFolder);

        jLabel1.setText("Select created analysis for comparson with selected folder:");

        historyTable.setModel(historyTableModel = new HistoryTableModel(logger));
        historyTable.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                historyTableFocusGained(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                historyTableFocusLost(evt);
            }
        });
        jScrollPane1.setViewportView(historyTable);

        loadYourOwnFile.setText("Load your own file");
        loadYourOwnFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadYourOwnFileActionPerformed(evt);
            }
        });

        compareButton.setText("Compare");
        compareButton.setEnabled(false);
        compareButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                compareButtonActionPerformed(evt);
            }
        });

        xmlIcon.setIcon(new javax.swing.ImageIcon(getClass().getResource("/cz/muni/fi/pb138/fsmda/gui/xml disc.png"))); // NOI18N

        jLabel3.setIcon(new javax.swing.ImageIcon(getClass().getResource("/cz/muni/fi/pb138/fsmda/gui/compare.png"))); // NOI18N

        compareProgressBar.setEnabled(false);

        cancelCompareButton.setText("Cancel");
        cancelCompareButton.setEnabled(false);
        cancelCompareButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelCompareButtonActionPerformed(evt);
            }
        });

        jLabel4.setIcon(new javax.swing.ImageIcon(getClass().getResource("/cz/muni/fi/pb138/fsmda/gui/folder small icon.png"))); // NOI18N

        jLabel5.setForeground(new java.awt.Color(102, 102, 102));
        jLabel5.setText("Selected folder:");

        selectFolderCompareButton.setText("Select Folder");
        selectFolderCompareButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                selectFolderCompareButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(xmlIcon)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addComponent(compareProgressBar, javax.swing.GroupLayout.DEFAULT_SIZE, 302, Short.MAX_VALUE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(cancelCompareButton, javax.swing.GroupLayout.PREFERRED_SIZE, 84, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addComponent(loadYourOwnFile)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(jLabel3)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(compareButton, javax.swing.GroupLayout.PREFERRED_SIZE, 115, javax.swing.GroupLayout.PREFERRED_SIZE))))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addComponent(jLabel1)
                        .addGap(0, 0, Short.MAX_VALUE))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel4)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addGap(10, 10, 10)
                                .addComponent(jLabel5)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(selectedFolderCompareLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(selectFolderCompareButton)
                                .addGap(0, 0, Short.MAX_VALUE)))))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel4)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(selectFolderCompareButton)
                        .addGap(6, 6, 6)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel5)
                            .addComponent(selectedFolderCompareLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 14, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 14, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 116, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGap(15, 15, 15)
                        .addComponent(loadYourOwnFile))
                    .addComponent(xmlIcon)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                        .addComponent(compareButton)
                        .addGap(16, 16, 16))
                    .addComponent(jLabel3, javax.swing.GroupLayout.Alignment.TRAILING))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(compareProgressBar, javax.swing.GroupLayout.DEFAULT_SIZE, 24, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(cancelCompareButton, javax.swing.GroupLayout.PREFERRED_SIZE, 24, javax.swing.GroupLayout.PREFERRED_SIZE)))
        );

        comparisionTab.addTab("Comparison", jPanel1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(comparisionTab)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(comparisionTab)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void selectFolder() {
        JFileChooser chooseFile = new JFileChooser();
        chooseFile.setDialogTitle("Select folder");
        chooseFile.setMultiSelectionEnabled(false);
        chooseFile.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        if (selectedFile != null) {
            chooseFile.setCurrentDirectory(selectedFile);
        }
        if (chooseFile.showDialog(this, "Select folder") == JFileChooser.APPROVE_OPTION) {
            File file = chooseFile.getSelectedFile();
            if (file == null || file.exists() == false) {
                JOptionPane.showMessageDialog(this, "Selected file doesn't exists!", "Wrong selection", JOptionPane.ERROR_MESSAGE);
                return;
            }
            if (file.isDirectory() == false) {
                JOptionPane.showMessageDialog(this, "Selected file isn't directory!", "Wrong selection", JOptionPane.ERROR_MESSAGE);
                return;
            }
            selectedFile = file;
            setEnabledCompareButton();
            SwingUtilities.invokeLater(new Runnable() {

                @Override
                public void run() {
                    selectedFolderText.setText(selectedFile.getPath());
                    selectedFolderCompareLabel.setText(selectedFile.getPath());
                    doAnalysisButton.setEnabled(true);
                }
            });
        }
    }
    private void chooseFolderButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_chooseFolderButtonActionPerformed
        selectFolder();
    }//GEN-LAST:event_chooseFolderButtonActionPerformed

    private void doAnalysisButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_doAnalysisButtonActionPerformed
        SwingUtilities.invokeLater(new Runnable() {

            @Override
            public void run() {
                analysisProgressBar.setIndeterminate(true);
                chooseFolderButton.setEnabled(false);
                cancelAnalysisButton.setEnabled(true);
                doAnalysisButton.setEnabled(false);
            }
        });
        analyze = new AnalyzeThread(MainJFrame.this, logger, selectedFile) {

            @Override
            public void finishAnalysis() {
                setButtonsAfterAnalysis();
                historyTableModel.refreshTableModel();
                //logger.info("File analysis was finished!");
            }
        };
        analyze.execute();
    }//GEN-LAST:event_doAnalysisButtonActionPerformed

    private void cancelAnalysisButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelAnalysisButtonActionPerformed
        setButtonsAfterAnalysis();
        if (analyze != null) {
            if (analyze.isDone() == false) {
                if (analyze.cancel(true));                    
                JOptionPane.showMessageDialog(this, "Analysis was canceled!", "Result of analysis", JOptionPane.WARNING_MESSAGE);
            }
        }        
    }//GEN-LAST:event_cancelAnalysisButtonActionPerformed

    private void loadYourOwnFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadYourOwnFileActionPerformed
        JFileChooser fileChooser = new JFileChooser();
        javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter() {
            @Override
            public boolean accept(File f) {
                if (f.isDirectory()) {
                    return true;
                }
                return f.getName().endsWith(".xml") == true;
            }

            @Override
            public String getDescription() {
                return "Just xml files";
            }
        };
        fileChooser.setFileFilter(filter);
        fileChooser.setApproveButtonText("Select");
        fileChooser.setName("Select .xml file with previous folder analysis");
        fileChooser.setDialogTitle("Select .xml file with previous folder analysis");
        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        int returnVal = fileChooser.showOpenDialog(this);
        File selectedFolderByFileChooser = fileChooser.getSelectedFile();
        if (returnVal == JFileChooser.APPROVE_OPTION && selectedFolderByFileChooser != null) {
            if (selectedFolderByFileChooser.exists() == false) {
                JOptionPane.showMessageDialog(this, "Selected file doesn't exists!", "Wrong file selection", JOptionPane.ERROR_MESSAGE);
                return;
            }
            if (selectedFolderByFileChooser.isDirectory() == true) {
                JOptionPane.showMessageDialog(this, "Selected file can't be a direcoty!", "Wrong file selection", JOptionPane.ERROR_MESSAGE);
                return;
            }
            //check if file can be accepted as file analysis !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            try {
                historyTableModel.addFile(selectedFolderByFileChooser);
            } catch (java.lang.IllegalArgumentException iae) {
                JOptionPane.showMessageDialog(this, iae.getMessage(), "Illegal argument exception", JOptionPane.INFORMATION_MESSAGE);
            }
        }
    }//GEN-LAST:event_loadYourOwnFileActionPerformed

    private void setEnabledCompareButton() {
        SwingUtilities.invokeLater(new Runnable() {

            @Override
            public void run() {
                compareButton.setEnabled(historyTable.getSelectedRow() >= 0 && selectedFile != null);
            }
        });
    }
    private void historyTableFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_historyTableFocusLost
        setEnabledCompareButton();
    }//GEN-LAST:event_historyTableFocusLost

    private void historyTableFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_historyTableFocusGained
        setEnabledCompareButton();
    }//GEN-LAST:event_historyTableFocusGained

    private void compareButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_compareButtonActionPerformed
        int selectedRow = historyTable.getSelectedRow();
        if (selectedRow < 0) {
            JOptionPane.showMessageDialog(this, "Select file from table.", "Can't compare", JOptionPane.INFORMATION_MESSAGE);
            return;
        }
        if (selectedFile == null) {
            JOptionPane.showMessageDialog(this, "You have to selected folder to compare!", "Can't compare", JOptionPane.INFORMATION_MESSAGE);
            return;
        }
        File selectedXMLfile = historyTableModel.getFile(selectedRow);
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                loadYourOwnFile.setEnabled(false);
                compareButton.setEnabled(false);
                compareProgressBar.setEnabled(true);
                cancelCompareButton.setEnabled(true);
                compareProgressBar.setIndeterminate(true);
            }
        });
        compare = new CompareThread(MainJFrame.this, logger, selectedFile, selectedXMLfile) {

            @Override
            public void finishAnalysis() {
                setButtonsAfterComparation();
            }
        };
        compare.execute();
    }//GEN-LAST:event_compareButtonActionPerformed

    private void cancelCompareButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelCompareButtonActionPerformed
        setButtonsAfterComparation();
        if (compare != null) {
            if (compare.isDone() == false) {
                if (compare.cancel(true));
                JOptionPane.showMessageDialog(this, "Comparison was canceled!", "Result of analysis", JOptionPane.WARNING_MESSAGE);
            }
        }        
    }//GEN-LAST:event_cancelCompareButtonActionPerformed

    private void selectFolderCompareButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_selectFolderCompareButtonActionPerformed
        selectFolder();
    }//GEN-LAST:event_selectFolderCompareButtonActionPerformed
    private void setButtonsAfterAnalysis() {
        SwingUtilities.invokeLater(new Runnable() {

            @Override
            public void run() {
                analysisProgressBar.setIndeterminate(false);
                chooseFolderButton.setEnabled(true);
                cancelAnalysisButton.setEnabled(false);
                doAnalysisButton.setEnabled(true);
            }
        });
    }

    private void setButtonsAfterComparation() {
        SwingUtilities.invokeLater(new Runnable() {

            @Override
            public void run() {
                compareProgressBar.setIndeterminate(false);
                loadYourOwnFile.setEnabled(true);
                compareButton.setEnabled(true);
                compareProgressBar.setEnabled(false);
                cancelCompareButton.setEnabled(false);
            }
        });
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(MainJFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(MainJFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(MainJFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(MainJFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        java.awt.EventQueue.invokeLater(new Runnable() {

            @Override
            public void run() {
                new MainJFrame().setVisible(true);
            }
        });
    }
    //selected File
    private File selectedFile = null;
    private AnalyzeThread analyze = null;
    private CompareThread compare = null;
    private HistoryTableModel historyTableModel = null;
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel analyseIcon;
    private javax.swing.JProgressBar analysisProgressBar;
    private javax.swing.JPanel analysisSelectedFolder;
    private javax.swing.JButton cancelAnalysisButton;
    private javax.swing.JButton cancelCompareButton;
    private javax.swing.JButton chooseFolderButton;
    private javax.swing.JButton compareButton;
    private javax.swing.JProgressBar compareProgressBar;
    private javax.swing.JTabbedPane comparisionTab;
    private javax.swing.JButton doAnalysisButton;
    private javax.swing.JLabel folderIcon;
    private javax.swing.JTable historyTable;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JButton loadYourOwnFile;
    private javax.swing.JToggleButton selectFolderCompareButton;
    private javax.swing.JLabel selectedFolderCompareLabel;
    private javax.swing.JLabel selectedFolderLabel;
    private javax.swing.JLabel selectedFolderText;
    private javax.swing.JLabel xmlIcon;
    // End of variables declaration//GEN-END:variables
}
